--[[
        Chat Buddy 
        Author:         Vycke Gilliam
        Version:        1.0: Initial release
        				1.1: Fix to handle the fact that if you were defaulting to
							 raid or party & leave said raid/party, the default UI
							 (Blizzard's stuff) would set you to /say -- even if you
							 just tried to Reply to someone.
						2.0: Fixed the fact that chatType (and the channel type) 
							 requires SetAttribute instead of just = on the frame/editBox
        Last Modified:  
				02/09/2005	-- initial release
				07/05/2005  -- 1.1
				12/15/2006	-- 2.0
]]

-- Configuration variables
    ChatBuddyFirstUse = 0;        -- Saved
    ChatBuddyEnabled = 0;         -- Saved - 1 = on, 0 = off
    ChatBuddyDefaultChannel = "g";	 -- '0' repeats the last-used channel on enter (inc. /o & /1)
-- Local variables
    local SavedChatBuddyHandler = nil;

-- on<something> functions

function ChatBuddy_OnLoad()
-- respond to saved variable load
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("PARTY_MEMBERS_CHANGED"); -- hoping this'll tell us when we leave a party/raid
end

function ChatBuddy_OnEvent()

    if (event == "VARIABLES_LOADED") then
    -- Saved variables loaded - check for first use and load defaults if necessary
        if (ChatBuddyFirstUse == 0) then
            ChatBuddyLoadDefaults();
        end
    -- Register command handler and new commands
		SlashCmdList["ChatBuddyCOMMAND"] = ChatBuddySlashHandler;
		SLASH_ChatBuddyCOMMAND1 = "/ChatBuddy";
		SLASH_ChatBuddyCOMMAND2 = "/cb";
		
		-- Hook the chat event handler
		if (ChatFrame_OpenChat ~= SavedChatBuddyHandler) then
			SavedChatBuddyHandler = ChatFrame_OpenChat;
			ChatFrame_OpenChat = ChatBuddy_OpenChat;
		end

		-- Report load
        ChatBuddyChatMsg("Chat Buddy loaded, type /ChatBuddy (or /cb) for useage");
		ChatBuddyDisplayStatus();
		
		--  initialize the sticky        
   		if (ChatBuddyEnabled == 1) then
			local box = DEFAULT_CHAT_FRAME.editBox;
			ChatBuddySetSticky(box);
			box:SetAttribute("chatType", box:GetAttribute("stickyType"));
			ChatEdit_UpdateHeader(box);
		end
	end 
end

function ChatBuddy_OpenChat(text, chatFrame)

	chatFrame = DEFAULT_CHAT_FRAME;
	box = chatFrame.editBox;

	if (ChatBuddyEnabled == 1) then
		if (( (box:GetAttribute("stickyType") == "PARTY") and (GetNumPartyMembers() == 0) ) or
			( (box:GetAttribute("stickyType") == "RAID") and (GetNumRaidMembers() == 0) )) then
			if (ChatBuddyDefaultChannel == "r" or ChatBuddyDefaultChannel == "raid") then
				ChatBuddyChatMsg("No longer in a raid, switching to /s as default");
			else -- party
				ChatBuddyChatMsg("No longer in a party, switching to /s as default");
			end
			ChatBuddyDefaultChannel = "s";
			if (box:GetAttribute("chatType") and box:GetAttribute("chatType") ~= "WHISPER") then
				box:SetAttribute("chatType", "SAY");
			end
		end

		ChatBuddySetSticky(box);

		if (box:GetAttribute("chatType")) then
			if (box:GetAttribute("chatType") ~= "WHISPER") then
				box:SetAttribute("chatType", box:GetAttribute("stickyType"));
			else -- whisper
				box.text = "";
				box.setText = 1;
			end
		end
	end
	
	box:Show();
end

function ChatBuddySetSticky(editBox)
	local type = editBox.chatType;
	local def = ChatBuddyDefaultChannel;

	if (ChatBuddyEnabled == 1) then
		
		if (def == "z") then
			-- last channel
			editBox:SetAttribute("stickyType", type);
		else 
			-- specific channel
			if ( def == "g") then
				editBox:SetAttribute("stickyType", "GUILD");
			elseif (def == "p") then
				editBox:SetAttribute("stickyType", "PARTY");
			elseif (def == "r" or def == "raid") then
				editBox:SetAttribute("stickyType", "RAID");
			elseif (def == "o") then
				editBox:SetAttribute("stickyType", "OFFICER");
			elseif (def == "s") then
				editBox:SetAttribute("stickyType", "SAY");
			elseif ((def >= "0") and (def <= "9")) then
				editBox:SetAttribute("stickyType", "CHANNEL");
				editBox:SetAttribute("channelTarget", def);
			else 
				-- regular method
				if ( ChatTypeInfo[type].sticky == 1 ) then
					editBox:SetAttribute("stickyType", type);
				end
			end
		end
	else
		-- this is the usual chat method
		if (type) then
			if ( ChatTypeInfo[type].sticky == 1 ) then
				editBox:SetAttribute("stickyType", type);
			end
		else
			editBox:SetAttribute("stickyType", "SAY");
		end
	end
end


-- Helper Functions
function ChatBuddySetEnabled(toggle)
    toggle = string.lower(toggle);
    if (toggle == "off") then
        ChatBuddyEnabled = 0;
        ChatBuddyChatMsg("Chat Buddy Off");
    else
        ChatBuddyEnabled = 1;
        ChatBuddyChatMsg("Chat Buddy On");
    end        
end


function ChatBuddySlashHandler(msg)
msg = string.lower(msg);

-- check for 2 arguments first
	local firsti, lasti, command, defchat = string.find (msg, "(%w+) (%w+)");
-- if there weren't 2, then command will be nil - check for one arg    
    if (command == nil) then
        firsti, lasti, command = string.find(msg, "(%w+)");
    end    
-- respond to commands
    if (command == nil) then
        ChatBuddyDisplayUseage();
    elseif (command == "enable") then
        ChatBuddySetEnabled("on");
    elseif (command == "disable") then
        ChatBuddySetEnabled("off");
    elseif (command == "status") then
    	ChatBuddyDisplayStatus();
    elseif (command == "set") then
        if (defchat == nil) then
           	ChatBuddyDisplayUseage();
        else
            ChatBuddyDefaultChannel = defchat;
            ChatBuddyDisplayStatus();
        end
    else
        ChatBuddyDisplayUseage();
    end
end

function ChatBuddyDisplayStatus()
local text = "Chat Buddy";
    if (ChatBuddyEnabled == 1) then
        text = text .. " On --";
        if (ChatBuddyDefaultChannel == "z") then
        	text = text .. " Using the last channel as default";
        else
        	text = text .. " Using /" .. ChatBuddyDefaultChannel .. " as default";
        end
    else
        text = text .. " Off";
    end
    ChatBuddyChatMsg(text);
end

function ChatBuddyDisplayUseage()
	local text = "Useage: \n/ChatBuddy <Enable | Disable | Status>\n";
    text = text .. "/ChatBuddy set <channelcode>  (i.e. g for /g, p for /p, r for /raid, z for last channel)";

    ChatBuddyChatMsg(text);

end

function ChatBuddyChatMsg(msg)
	if( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage(msg);
	end
end

function ChatBuddyLoadDefaults()
local text;
    ChatBuddyFirstUse = 1;        
    ChatBuddyEnabled = 1;         
    ChatBuddyDefaultChannel = "g";

    text = "ChatBuddy loading defaults\n";
    text = text .. "type /ChatBuddy for commands";

    ChatBuddyChatMsg(text);
end
